package daemonset

import (
	"encoding/json"

	"github.com/fusor/openshift-velero-plugin/velero-plugins/common"
	"github.com/heptio/velero/pkg/plugin/velero"
	"github.com/sirupsen/logrus"
	appsv1API "k8s.io/api/apps/v1"
	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
)

// RestorePlugin is a restore item action plugin for Velero
type RestorePlugin struct {
	Log logrus.FieldLogger
}

// AppliesTo returns a velero.ResourceSelector that applies to daemonsets
func (p *RestorePlugin) AppliesTo() (velero.ResourceSelector, error) {
	return velero.ResourceSelector{
		IncludedResources: []string{"daemonsets.apps"},
	}, nil
}

// Execute action for the restore plugin for the daemonset resource
func (p *RestorePlugin) Execute(input *velero.RestoreItemActionExecuteInput) (*velero.RestoreItemActionExecuteOutput, error) {
	p.Log.Info("[daemonset-restore] Entering DaemonSet restore plugin")

	daemonSet := appsv1API.DaemonSet{}
	itemMarshal, _ := json.Marshal(input.Item)
	json.Unmarshal(itemMarshal, &daemonSet)
	p.Log.Infof("[daemonset-restore] daemonset: %s", daemonSet.Name)

	backupRegistry, registry, err := common.GetSrcAndDestRegistryInfo(input.Item)
	if err != nil {
		return nil, err
	}
	common.SwapContainerImageRefs(daemonSet.Spec.Template.Spec.Containers, backupRegistry, registry, p.Log)
	common.SwapContainerImageRefs(daemonSet.Spec.Template.Spec.InitContainers, backupRegistry, registry, p.Log)

	var out map[string]interface{}
	objrec, _ := json.Marshal(daemonSet)
	json.Unmarshal(objrec, &out)

	return velero.NewRestoreItemActionExecuteOutput(&unstructured.Unstructured{Object: out}), nil
}
